@@ -1040,7 +1040,7 @@ static void __init sscape_pnp_init_hw(sscape_info* devc)
 	i = sscape_read_host_ctrl( devc );
 	
 	if ( (i & 0x0F) >  7 ) {
-		printk(KERN_ERR "sscape: scope.cod faild\n");
+		printk(KERN_ERR "sscape: /*(DEBLOBBED)*/ faild\n");
 		sscape_pnp_free_dma(devc);
 		return;
 	}
